
# q-expansion of newform 1728.3.h.e, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1728
weight = 3
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 0, -2], 1], [[0, 6, 0, -3], 1], [[-6, 0, 12, 0], 1]]

hecke_ring_character_values = [[703, [1, 0, 0, 0]], [325, [-1, 0, 0, 0]], [1217, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 1, 0], [0, 0, 1, 0], [3, 0, 0, 0], [0, 0, 0, 1], [0, -3, 0, 0], [0, -1, 0, 0], [0, 0, 0, 1], [0, 0, 4, 0], [0, 0, 7, 0], [0, 0, 0, 5], [0, -21, 0, 0], [0, -2, 0, 0], [0, 0, 0, 4], [0, 0, -13, 0], [66, 0, 0, 0], [0, 0, 0, -6], [0, 22, 0, 0], [0, 0, 0, -13], [29, 0, 0, 0], [0, 0, -16, 0], [99, 0, 0, 0], [0, -72, 0, 0], [31, 0, 0, 0], [0, 0, 11, 0], [0, 0, -12, 0], [129, 0, 0, 0], [0, 0, 0, -17], [0, -45, 0, 0], [0, 0, -7, 0], [153, 0, 0, 0], [0, -120, 0, 0], [0, 68, 0, 0], [0, 0, 15, 0], [0, 0, -9, 0], [0, 0, 0, -10], [0, -55, 0, 0], [0, 0, 0, 14], [0, 0, 13, 0], [153, 0, 0, 0], [0, 0, 0, 16], [0, 0, 0, 11], [-271, 0, 0, 0], [0, 0, 21, 0], [0, 0, 25, 0], [0, 157, 0, 0], [0, 0, 68, 0], [150, 0, 0, 0], [0, 0, 0, 8], [0, -150, 0, 0], [0, 0, 0, -37], [122, 0, 0, 0], [102, 0, 0, 0], [0, 57, 0, 0], [0, 0, 0, -33], [0, 0, 48, 0], [0, 0, -33, 0], [0, 0, 0, 2], [0, -99, 0, 0], [0, 149, 0, 0], [0, 0, -52, 0], [0, -143, 0, 0], [0, 0, 0, 35], [193, 0, 0, 0], [0, 0, -93, 0], [0, 20, 0, 0], [-422, 0, 0, 0], [39, 0, 0, 0], [0, 0, 0, 11], [0, 237, 0, 0], [0, 0, 0, 26], [0, 0, -117, 0], [0, 0, 0, -59], [0, -206, 0, 0], [0, 0, 0, -55], [0, 0, -1, 0], [0, 0, 0, -19], [0, 330, 0, 0], [-649, 0, 0, 0], [-198, 0, 0, 0], [0, 0, 0, -76], [0, 0, 0, 56], [331, 0, 0, 0], [0, 0, -99, 0], [-102, 0, 0, 0], [0, 345, 0, 0], [203, 0, 0, 0], [0, 0, 135, 0], [0, 0, -97, 0], [-381, 0, 0, 0], [0, 0, 0, -6], [0, 0, 44, 0], [-759, 0, 0, 0], [0, 346, 0, 0], [0, 0, 0, -77], [0, 0, -31, 0], [0, 105, 0, 0], [0, -341, 0, 0], [0, 0, 0, 8], [0, -259, 0, 0], [0, 0, -153, 0], [-561, 0, 0, 0], [0, -132, 0, 0], [0, 203, 0, 0], [610, 0, 0, 0], [-483, 0, 0, 0], [0, 414, 0, 0], [0, 0, 0, -2], [5, 0, 0, 0], [0, 0, 148, 0], [0, 0, 0, 30], [0, -159, 0, 0], [0, 209, 0, 0], [0, 0, -7, 0], [0, 543, 0, 0], [0, 374, 0, 0], [0, 0, 0, -64], [0, 0, -87, 0], [-279, 0, 0, 0], [0, 0, 0, 33], [-77, 0, 0, 0], [0, 0, 220, 0], [-714, 0, 0, 0], [0, -502, 0, 0], [0, 0, -27, 0], [0, 0, 0, -2], [0, 0, 0, -90], [0, 0, 33, 0], [0, 0, 0, -77], [0, -143, 0, 0], [0, 0, 0, 35], [0, 0, -281, 0], [0, 0, 0, -66], [0, 0, 0, 0], [-55, 0, 0, 0], [0, 0, -244, 0], [0, 482, 0, 0], [0, 0, 253, 0], [0, -300, 0, 0], [0, -125, 0, 0], [0, 0, 52, 0], [0, 0, -53, 0], [1026, 0, 0, 0], [0, 0, 0, -82], [0, 0, 0, 109], [0, 0, 0, 66], [0, -405, 0, 0], [0, -374, 0, 0], [0, 0, 0, 2], [0, 0, 0, -142], [0, 171, 0, 0], [0, 269, 0, 0], [0, 0, 0, -75], [0, 187, 0, 0], [0, 0, 0, -44], [0, 0, -249, 0], [0, -54, 0, 0], [-713, 0, 0, 0], [0, 0, -221, 0], [825, 0, 0, 0], [0, -438, 0, 0], [0, 0, -333, 0], [-729, 0, 0, 0], [0, -318, 0, 0], [0, 0, 0, -24], [0, 0, 69, 0], [0, 0, 0, 117], [-209, 0, 0, 0], [0, 0, 36, 0], [1437, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -98], [-626, 0, 0, 0], [0, 0, 187, 0], [0, -585, 0, 0], [0, 641, 0, 0], [0, 0, 161, 0], [0, 0, 395, 0], [0, 0, 0, -79], [0, 0, -185, 0], [831, 0, 0, 0], [0, 0, 0, 57], [0, -588, 0, 0], [0, 0, 0, -99], [0, 0, -57, 0], [0, 0, 0, 99], [0, -605, 0, 0], [1045, 0, 0, 0], [0, 0, 0, -71], [-322, 0, 0, 0], [141, 0, 0, 0], [0, 208, 0, 0], [0, 0, -275, 0], [1254, 0, 0, 0], [0, -186, 0, 0], [1765, 0, 0, 0], [0, 0, 0, -100], [0, -894, 0, 0], [0, 0, 0, 206], [0, 0, -209, 0], [0, 0, 265, 0], [0, 0, 0, 5], [-1247, 0, 0, 0], [-33, 0, 0, 0], [0, 0, 308, 0], [0, 0, -404, 0], [2031, 0, 0, 0], [0, -516, 0, 0], [0, -1265, 0, 0], [-1519, 0, 0, 0], [0, 0, 68, 0], [0, 0, -237, 0], [2349, 0, 0, 0], [0, 0, 0, -84], [-121, 0, 0, 0], [0, 0, 188, 0], [0, 258, 0, 0], [0, 0, 0, 66], [0, 0, 220, 0], [0, 0, 0, -145], [0, 0, 444, 0], [0, -744, 0, 0], [0, 0, 88, 0], [759, 0, 0, 0], [0, 0, 0, -220], [0, 414, 0, 0], [0, 0, 0, -63], [0, 0, -100, 0], [-1566, 0, 0, 0], [0, 0, 0, 26], [0, 836, 0, 0], [0, 0, -460, 0], [0, 960, 0, 0], [0, 1456, 0, 0], [0, 0, 0, -16], [478, 0, 0, 0], [0, 0, 88, 0], [2103, 0, 0, 0], [0, 0, 0, -114], [-1122, 0, 0, 0], [0, 872, 0, 0], [0, 0, -531, 0], [0, 0, 0, -135], [0, -705, 0, 0], [0, 0, 0, 37], [0, 0, 143, 0], [-1707, 0, 0, 0], [0, -1471, 0, 0], [0, 0, 0, -153], [0, 0, 0, 0], [0, 681, 0, 0], [0, 0, 0, -274], [751, 0, 0, 0], [0, 0, 381, 0], [489, 0, 0, 0], [0, 0, 0, 75], [0, 1478, 0, 0], [0, 0, -361, 0], [335, 0, 0, 0], [0, 0, 11, 0], [1425, 0, 0, 0], [0, 0, 0, 66], [0, 0, 0, 73], [0, -324, 0, 0], [0, -506, 0, 0], [0, 0, -604, 0], [0, 1329, 0, 0], [0, -1373, 0, 0], [0, 0, -292, 0], [0, 0, 0, -95], [0, 1396, 0, 0], [-1661, 0, 0, 0], [0, 0, -3, 0], [2735, 0, 0, 0], [0, 0, 165, 0], [2079, 0, 0, 0], [0, 0, 0, 317], [-2995, 0, 0, 0], [-3246, 0, 0, 0], [0, 0, 0, 162], [0, 0, 281, 0], [0, 0, 0, -45], [0, 0, 0, 267], [0, -392, 0, 0], [0, 0, 0, -216], [209, 0, 0, 0], [0, 0, 77, 0], [0, 0, 324, 0], [0, -318, 0, 0], [0, 0, 524, 0], [-1623, 0, 0, 0], [0, 618, 0, 0], [-3579, 0, 0, 0], [0, 0, 0, 213], [0, 0, 444, 0], [0, 0, 293, 0], [0, 0, 111, 0], [-81, 0, 0, 0], [0, -1042, 0, 0], [1234, 0, 0, 0], [0, 0, -115, 0], [0, 0, 605, 0]]
